/*
 * Copyright:      Jan Homann
 * Author   :      Jan Homann 
 * Version  :      1.0 
 * Date     :	   20.10.2015
 * Source   :      -Datasheet (MEGA32)
 */

#include <avr/io.h>
#include <avr/sfr_defs.h>
#include <avr/interrupt.h>
#include "spi.h"


volatile uint8_t data[8];

void spi_master_init(uint8_t prescaler, uint8_t clock_option)
{	
	/* ONLY FOR MEGA32! */
	#if defined(__AVR_ATmega32__)
	{
		/* set MOSI and SCK output, all others input */
		MOSI_DDR |= (1<<MOSI_BIT);
		SCK_DDR	 |=	(1<<SCK_BIT);
		SS_DDR	 |= (1<<SS_BIT);
	
		/* enable SPI, Master */
		SPCR |= (1<<SPE) | (1<<MSTR);
		
		/* clear the bits (for multi init) */
		SPCR &= ~((1<<SPI2X) | (1<<SPR0) | (1<<SPR1)); // fclk/4
	
		/* config SPI Bus Speed fclk/prescaler */
		switch (prescaler)
		{
			case 2:   {SPCR |=  (1<<SPI2X);							 };break;
			case 8:   {SPCR |=  ((1<<SPI2X) | (1<<SPR0));			 };break;
			case 32:  {SPCR |=  ((1<<SPI2X) | (1<<SPR1));			 };break;
			case 64:  {SPCR |=  ((1<<SPI2X) | (1<<SPR1) | (1<<SPR0));};break;
			default:  {SPCR |=  (1<<SPI2X);							 };break;
		}
	}
	#endif	
	
	/* ONLY FOR MEGA32U4! */
	#if defined (__AVR_ATmega32U4__)
	{
			/* set MOSI and SCK output, all others input */
			MOSI_DDR	|= (1<<MOSI_BIT);
			SCK_DDR		|=	(1<<SCK_BIT);
			SS_DDR		|= (1<<SS_BIT);	   // software slave select signal
			SS_HA_DDR	|= (1<<SS_HA_BIT); // hardware slave select signal
			
			/* enable SPI, Master */
			SPCR |= ((1<<SPE) | (1<<MSTR));			
			
			/* config "Clock Polarity and "Clock Phase" */
			switch(clock_option)
			{
				case 0 : {SPCR &= ~((1<<CPOL) | (1<<CPHA)); }break; // mode 1
				case 1 : {SPCR |=  (1<<CPHA);				}break; // mode 2
				case 2 : {SPCR |=  (1<<CPOL);				}break; // mode 3
				case 3 : {SPCR |=  ((1<<CPOL) | (1<<CPHA));	}break; // mode 4
			}
			
			/* config SPI Bus Speed fclk/prescaler */
			switch (prescaler)
			{
				case 2:   {SPSR |= (1<<SPI2X);								  }break; // prescaler 2
				case 8:   {SPSR |= (1<<SPI2X); SPCR |= (1<<SPR0);			  }break; // prescaler 8
				case 32:  {SPSR |= (1<<SPI2X); SPCR |= (1<<SPR1);			  }break; // prescaler 32
				case 64:  {SPSR |= (1<<SPI2X); SPCR |= (1<<SPR1) | (1<<SPR0); }break; // prescaler 64
				default:  {SPSR |= (1<<SPI2X);								  }break; // default = 2
			}
	}
	#endif
}

void spi_slave_init(uint8_t prescaler)
{
	/* ONLY FOR MEGA32! */
	#if defined(__AVR_ATmega32__)
	{
		/* set MISO output, all others input */
		MISO_DDR |= (1<<MISO_BIT);
		
		/* enable SPI */
		SPCR |= (1<<SPE);

		/* clear the bits (for multi init) */
		SPCR &= ~((1<<SPI2X) | (1<<SPR0) | (1<<SPR1)); // fclk/4

		/* config SPI Bus Speed fclk/prescaler */
		switch (prescaler)
		{
			case 4:   {SPCR &= ~((1<<SPR1) | (1<<SPR0)); };break;
			case 16:  {SPCR |=  (1<<SPR0);				 };break;
			case 64:  {SPCR |=  (1<<SPR1);				 };break;
			case 128: {SPCR |=  ((1<<SPR0) | (1<<SPR1)); };break;
			default:  {SPCR &= ~((1<<SPR1) | (1<<SPR0)); };break;
		}
	}
	#endif	
	
	/* ONLY FOR MEGA32U4! */
	#if defined (__AVR_ATmega32U4__)
	{
		/* set MISO output, all others input */
		MISO_DDR |= (1<<MISO_BIT);
		
		/* enable SPI */
		SPCR |= (1<<SPE);

		/* clear the bits (for multi init) */
		SPSR &= ~(1<<SPI2X);
		SPCR &= ~((1<<SPR0) | (1<<SPR1)); // fclk/4

		/* config SPI Bus Speed fclk/prescaler */
		switch (prescaler)
		{
			case 4:   {SPCR &= ~((1<<SPR1) | (1<<SPR0));	SPSR &= ~(1<<SPI2X); }break;
			case 16:  {SPCR |=  (1<<SPR0);										 }break;
			case 64:  {SPCR |=  (1<<SPR1);										 }break;
			case 128: {SPCR |=  ((1<<SPR0) | (1<<SPR1));						 }break;
			default:  {SPCR &= ~((1<<SPR1) | (1<<SPR0));						 }break;
		}
	}
	#endif
}

void spi_master_transmit(uint8_t cData)
{
	/* start transmission */
	SPDR = cData;
	
	/* wait for transmission complete */
	while(!(SPSR & (1<<SPIF)));
}

uint8_t spi_receive(void)
{	
	/* Wait for reception complete */
	//while(!(SPSR & (1<<SPIF)));				

	/* return data register */
	return SPDR;	
}

void spi_enable_chip(void)
{
	SS_PORT |= (1<<SS_BIT);
}

void spi_disable_chip(void)
{
	SS_PORT &= ~(1<<SS_BIT);
}

/* byte received */
ISR(SPI_STC_vect)
{
}